Extension { #name : 'TranscriptStreamPortable' }

{ #category : 'initialization' }
TranscriptStreamPortable class >> initialize [
	"self initialize"

	Globals at: #Transcript put: self new.
	 "world write permission"
	(Globals associationAt: #Transcript) objectSecurityPolicy: nil

]

{ #category : 'Adding' }
TranscriptStreamPortable >> _nextPut: anObject [
"see PrintStream>>_nextPut:"

self nextPut: anObject

]

{ #category : 'Character writing' }
TranscriptStreamPortable >> crlf [
"Append a carriage return character followed by a line feed character to the receiver."

self nextPut: Character cr; nextPut: Character lf

]

{ #category : 'Character writing' }
TranscriptStreamPortable >> crtab [
"Append a return character, followed by a single tab character, to the
receiver."

self nextPut: Character cr.
self nextPut: Character tab

]

{ #category : 'Character writing' }
TranscriptStreamPortable >> crtab: anInteger [
"Append a return character, followed by anInteger tab characters, to the
receiver."

self nextPut: Character cr.
anInteger timesRepeat: [self nextPut: Character tab]

]

{ #category : 'private' }
TranscriptStreamPortable >> endEntry [

	self mutex critical: [ | contents |
		contents := self stream contents.
		self stream reset.
		(contents size > 1 and: [ (contents at: 1) = Character lf])
			ifTrue: [
				"gciLogServer tacks a lf on end of previous contents"
				contents := contents copyFrom: 2 to: contents size].
		GsFile gciLogServer: contents ].

]

{ #category : 'ANSI' }
TranscriptStreamPortable >> flush [

	self endEntry

]

{ #category : 'private' }
TranscriptStreamPortable >> mutex [
	| tmps |
	tmps := SessionTemps current  .
	^ (tmps at: #TranscriptStream_SessionMutex otherwise: nil)
	     ifNil:[ tmps at: #TranscriptStream_SessionMutex
			put: Semaphore forMutualExclusion ]

]

{ #category : 'Adding' }
TranscriptStreamPortable >> nextPut: anObject [

	self mutex critical: [ ^self stream nextPut: anObject ]

]

{ #category : 'Adding' }
TranscriptStreamPortable >> nextPutAll: aCollection [

	self mutex critical: [ ^self stream nextPutAll: aCollection ]

]

{ #category : 'Adding' }
TranscriptStreamPortable >> show: anObject [

	self nextPutAll: anObject printString.
	self endEntry

]

{ #category : 'Character writing' }
TranscriptStreamPortable >> space: anInteger [
"Append anInteger space characters to the receiver."

anInteger timesRepeat: [self space]

]

{ #category : 'private' }
TranscriptStreamPortable >> stream [

	| tmps |
	tmps := SessionTemps current  .
	^( tmps at: #'TranscriptStream_SessionStream' otherwise: nil)
		ifNil:[ tmps at: #'TranscriptStream_SessionStream'
				put: (WriteStream on: String new) ]


]

{ #category : 'Character writing' }
TranscriptStreamPortable >> tab: anInteger [
"Append anInteger tab characters to the receiver."

anInteger timesRepeat: [self tab]

]
{ #category : 'Adding' }
TranscriptStreamPortable >> nextPutAllBytes: aCharacterCollection [
  ^ self shouldNotImplement: #nextPutAllBytes: 
]
{ #category : 'Testing' }
TranscriptStreamPortable >> atEnd [
  ^ self shouldNotImplement: #atEnd
]
{ #category : 'Accessing' }
TranscriptStreamPortable >> next [
  ^ self shouldNotImplement: #next
]

